@@ -14,7 +14,8 @@ class FormError(Exception):
 
 
 FormElement = collections.namedtuple(
-    "FormElement", ["getter", "setter", "widgets", "subelements", "register_observer"]
+    "FormElement",
+    ["getter", "setter", "resetter", "widgets", "subelements", "register_observer"],
 )
 
 
@@ -59,6 +60,7 @@ class Form:
         vertically_place_labels=False,
         use_sliders=False,
         preconstruct_array_items=0,
+        sorter=sorted,
     ):
         """Create a form with Jupyter widgets from a JSON schema
 
@@ -80,6 +82,9 @@ class Form:
             subschemas for each array item, this can be a performance optimization
             trading construction time vs. usage delay.
         :type preconstruct_array_items: int
+        :param sorter:
+            A function that is used to sort the keys in a dictionary. Defaults to
+            the built-in sorted, but is no-op if sorted raises a TypeError.
         """
         # Make sure that the given schema is valid
         filename = os.path.join(
@@ -95,6 +100,7 @@ class Form:
         self.vertically_place_labels = vertically_place_labels
         self.use_sliders = use_sliders
         self.preconstruct_array_items = preconstruct_array_items
+        self.sorter = sorter
 
         # Store a list of registered observers to add them to runtime-generated widgets
         self._observers = []
@@ -106,6 +112,7 @@ class Form:
         self,
         getter=lambda: None,
         setter=lambda _: None,
+        resetter=lambda: None,
         widgets=[],
         subelements=[],
         register_observer=lambda h, n, t: None,
@@ -113,6 +120,7 @@ class Form:
         return FormElement(
             getter=getter,
             setter=setter,
+            resetter=resetter,
             widgets=widgets,
             subelements=subelements,
             register_observer=register_observer,
@@ -202,8 +210,16 @@ class Form:
         for prop, subschema in schema["properties"].items():
             elements[prop] = self._construct(subschema, label=prop)
 
+        # Apply sorting to the keys
+        keys = schema["properties"].keys()
+        try:
+            keys = self.sorter(keys)
+        except TypeError:
+            # If the keys cannot be compared, we stick to the original order
+            pass
+
         # If this is not the root document, we wrap this in an Accordion widget
-        widget_list = sum((e.widgets for e in elements.values()), [])
+        widget_list = sum((elements[k].widgets for k in keys), [])
         if not root:
             widget_list = self._wrap_accordion(widget_list, schema, label=label)
 
@@ -215,9 +231,14 @@ class Form:
             for e in elements.values():
                 e.register_observer(h, n, t)
 
+        def _resetter():
+            for e in elements.values():
+                e.resetter()
+
         return self.construct_element(
             getter=lambda: {p: e.getter() for p, e in elements.items()},
             setter=_setter,
+            resetter=_resetter,
             widgets=widget_list,
             subelements=elements,
             register_observer=_register_observer,
@@ -251,15 +272,6 @@ class Form:
         if tooltip is not None:
             widget.tooltip = tooltip
 
-        # Apply a potential default
-        if "default" in schema:
-            widget.value = schema["default"]
-        else:
-            if "minimum" in schema:
-                widget.value = schema["minimum"]
-            if "maximum" in schema:
-                widget.value = schema["maximum"]
-
         # Apply potential constant values without generating a widget
         if "const" in schema:
             return self.construct_element(getter=lambda: schema["const"])
@@ -287,6 +299,16 @@ class Form:
                     f"Value '{_d}' does not match the specified pattern '{schema['pattern']}'"
                 )
 
+        def _resetter():
+            # Apply a potential default
+            if "default" in schema:
+                widget.value = schema["default"]
+            else:
+                if "minimum" in schema:
+                    widget.value = schema["minimum"]
+                if "maximum" in schema:
+                    widget.value = schema["maximum"]
+
         def _getter():
             if not pattern_checker(widget.value):
                 # We will have to see whether or not throwing is a good idea here
@@ -296,6 +318,9 @@ class Form:
 
             return widget.value
 
+        # Trigger generation of defaults in construction
+        _resetter()
+
         # Make sure the widget adapts to the outer layout
         widget.layout = ipywidgets.Layout(width="100%")
 
@@ -307,6 +332,7 @@ class Form:
         return self.construct_element(
             getter=_getter,
             setter=_setter,
+            resetter=_resetter,
             widgets=[box_type(box)],
             register_observer=_register_observer,
         )
@@ -401,6 +427,22 @@ class Form:
                 if element_size == schema["maxItems"]:
                     return
 
+            def trigger_observers():
+                # Adding or removing an entry to this widget should trigger all value change handlers.
+                # As we do not have a proper widget to register the handler, we trigger it
+                # ourselves. This should make proper use of traitlets.
+                for h, n, t in self._observers:
+                    if t == "change" and (n is traitlets.All or "value" in as_tuple(n)):
+                        h(
+                            {
+                                "name": "value",
+                                "old": {},
+                                "new": {},
+                                "owner": None,
+                                "type": "change",
+                            }
+                        )
+
             # A new element should only be generated if we do not have an excess
             # one stored in the elements list
             if element_size == len(elements):
@@ -423,27 +465,6 @@ class Form:
                     icon="arrow-down", layout=ipywidgets.Layout(width="33%")
                 )
 
-                def trigger_observers():
-                    # Adding or removing an entry to this widget should trigger all value change handlers.
-                    # As we do not have a proper widget to register the handler, we trigger it
-                    # ourselves. This should make proper use of traitlets.
-                    for h, n, t in self._observers:
-                        if t == "change" and (
-                            n is traitlets.All or "value" in as_tuple(n)
-                        ):
-                            h(
-                                {
-                                    "name": "value",
-                                    "old": {},
-                                    "new": {},
-                                    "owner": None,
-                                    "type": "change",
-                                }
-                            )
-
-                # We trigger observers upon adding
-                trigger_observers()
-
                 def remove_entry(b):
                     nonlocal element_size
 
@@ -501,21 +522,23 @@ class Form:
                     ]
                 )
 
+                # Modify recelem to our needs
+                elemdict = recelem._asdict()
+                elemdict["widgets"] = [array_entry_widget]
+
                 # Insert this into the elements list
-                elements.append(
-                    self.construct_element(
-                        getter=recelem.getter,
-                        setter=recelem.setter,
-                        widgets=[array_entry_widget],
-                        subelements=recelem.subelements,
-                        register_observer=recelem.register_observer,
-                    )
-                )
+                elements.append(self.construct_element(**elemdict))
+
+            # Maybe reset it to the default
+            elements[element_size].resetter()
 
             # Regardless of whether we actually constructed an element or whether
             # we are reusing an existing one - we need to increase the size now
             element_size = element_size + 1
 
+            # Trigger observes when item is added regardless whether it was preconstructed
+            trigger_observers()
+
             update_widget()
 
         button.on_click(add_entry)
@@ -547,13 +570,15 @@ class Form:
             for e in elements:
                 e.register_observer(h, n, t)
 
-        # If a default was specified, we now set it
-        if "default" in schema:
-            _setter(schema["default"])
+        def _resetter():
+            # If a default was specified, we now set it
+            if "default" in schema:
+                _setter(schema["default"])
 
         return self.construct_element(
             getter=lambda: [h.getter() for h in elements[:element_size]],
             setter=_setter,
+            resetter=_resetter,
             widgets=wrapped_vbox,
             subelements=elements,
             register_observer=_register_observer,
@@ -609,6 +634,10 @@ class Form:
                 except jsonschema.ValidationError:
                     pass
 
+        def _resetter():
+            for e in elements:
+                e.resetter()
+
         def _register_observer(h, n, t):
             selector.observe(h, names=n, type=t)
             for e in elements:
@@ -617,6 +646,7 @@ class Form:
         return self.construct_element(
             getter=lambda: elements[names.index(selector.value)].getter(),
             setter=_setter,
+            resetter=_resetter,
             widgets=[ipywidgets.VBox(children=title + [widget])],
             subelements=elements,
             register_observer=_register_observer,
